/*****************************************************************************
*                                                                            *
*      COMMON OCEANOGRAPHIC DATA ACCESS SYSTEM (CODAS)                       *
*                                                                            *
*      WRITTEN BY:  RAMON CABRERA, ERIC FIRING, and JULIE RANADA             *
*                   JOINT INSTITUTE FOR MARINE AND ATMOSPHERIC RESEARCH      *
*                   1000 POPE ROAD  MSB 404                                  *
*                   HONOLULU, HI 96822                                       *
*                                                                            *
*      VERSION:     3.00                                                     *
*                                                                            *
*      DATE:        APRIL 1989                                               *
*                                                                            *
*****************************************************************************/
/*

    FILE:  io_nc.h

           Comment-stripping file access routines.

*/
#ifndef io_nc_included
#define io_nc_included
#ifndef dbhost_included
#include "dbhost.h"   /* PROTOTYPE_ALLOWED */
#endif
#if PROTOTYPE_ALLOWED
int getc_nc(FILE *fp);
int getword_nc(FILE *fp, char *buf, int bufsize);
int getwords_nc(FILE *fp, char *buf, int bufsize, int nwords);
int get2eol_nc(FILE *fp, char *buf, int bufsize);
void strip_comments(FILE *fp_in, FILE *fp_out);
char *get_quotation(FILE *fp, char *buf, int bufsize);
#else
int getc_nc();
int getword_nc();
int getwords_nc();
int get2eol_nc();
void strip_comments();
char *get_quotation();
#endif
#endif /* ifndef io_nc_included */

